/*	sms - application for sending, receiving and management of short message
 *
 *	Authors: YIN Huasong, WU Yu <yu.wu@orange-ftgroup.com> 
 *	
 *	This software and associated documentation files (the "Software") 
 *	are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *	All Rights Reserved. 
 *
 *	A copyright license is hereby granted for redistribution and use of 
 *	the Software in source and binary forms, with or without modification, 
 *	provided that the following conditions are met: 
 *	- Redistributions of source code must retain the above copyright notice, 
 *	this copyright license and the following disclaimer. 
 *  - Redistributions in binary form must reproduce the above copyright 
 * 	notice, this copyright license and the following disclaimer in the 
 *	documentation and/or other materials provided with the distribution. 
 *	- Neither the name of LiPS  nor the names of its Members may be used 
 *	to endorse or promote products derived from the Software without 
 *	specific prior written permission. 
 *
 *	A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *	to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *	distribute any implementation compliant with the any specification adopted 
 *	by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *	and non-discriminatory terms and conditions and under reciprocity, as 
 *	regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *	"AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *	THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *	AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *	ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *	POSSIBILITY OF SUCH DAMAGE. 
 */


#ifndef __MSG_SMS_CALLBACKS_H__
#define __MSG_SMS_CALLBACKS_H__

#include <message.h>
#include <glib.h>

#define PREFIX            getenv("SMS_GRAPHIC_PATH")
#define ABOOK_DATA_PATH   getenv("ABOOK_DATA_PATH")
#define IMAGEPATH         "pixmaps/"
#define UI_DES            "xml/ui-des.xml"
#define RC                "rc/testgtkrc"



/* select row */
typedef struct
{
  gint msgid;
  gint fid;
  gint root_fid;
  GArray *garray;
  /* GArray *garray_abook; */
} msg_sms_test_t;

/* cols of tree_view */
typedef struct
{
  gchar name[60];
  gint id;
  GdkPixbuf *icon;
} msg_sms_tree_col_t;

/* control flags */
typedef struct
{
  gboolean multi_select_flag;   /* flag of multiselect */     
  gboolean sort_date_flag;
  gboolean view_new_flag;
  gboolean start_from_abook_flag;
  gboolean dbus_invoke_flag;
  gboolean create_flag;
  gboolean open_flag;
} msg_sms_flag_t;

/* refresh tree_view about folders */
typedef struct
{
  GtkListStore *sms_list_store;
  GtkListStore *folder_list_store;
  GtkTreeView *sms_list_tree_view;
  GtkTreeView *folder_list_tree_view;
  GtkTreeView *msg_list_tree_view;
} msg_sms_tree_view_t;

/* control tree_view path */
typedef struct
{
  gchar *folder_path;
  gchar *sms_list_path;
  gint sms_number;
} msg_sms_tree_view_path_t;

/* create sms */
typedef struct
{
  GtkTextBuffer *buffer_contact;
  GtkTextBuffer *buffer_content;
} msg_sms_create_text_view_t;

/* no use */
typedef struct
{
  GtkWidget *window;
  gboolean have_softkeys;
  gint num_softkeys;
  GtkActionGroup *action_group;
  GtkUIManager *ui_manager;
  GtkWidget *menu_left;
  GtkWidget *menu_right;
} self_t;

enum
{
  SK_LEFT = 0,
  SK_RIGHT = 1,
  SK_MAX = 2
};

/* the cols of tree_view */
enum
{
  COLUMN_CHECK_BUTTON = 0,
  COLUMN_ICON,
  COLUMN_ID,
  COLUMN_TEXT,
  COLUMN_NUMS
};

/* sms create and setting, like the folder's id */
enum
{
  MSG_SMS_CREATE = 100,
  MSG_SMS_SETTING
};

/* define msg sms the window's number */
enum
{
  MSG_ROOT = 0,
  MSG_SMS_FOLDERS = 1,
  MSG_SMS_CREATE_OR_FOLDERLIST = 2,
  MSG_SMS_VIEW_FROM_FOLDER = 3
};

/* extern variable */
msg_sms_create_text_view_t  textview;
msg_sms_tree_col_t          tree_col;
msg_sms_test_t              msgtest;
msg_sms_flag_t              sms_flag;
msg_sms_tree_view_t         sms_treeview;
msg_sms_tree_view_path_t    sms_path;


gboolean file_exists (const char *filename);

/* the function show checkboxs when multi_select_flag == TRUE */
void actived_renderer_toggle (GtkCellRendererToggle *renderer, gchar *path,
                              GtkTreeView *tree_view);

/* create scrolled_window */
GtkWidget *make_swindow (GtkWidget *vbox);

/* show the pix */
GdkPixbuf *get_icon (gchar *image_name);

/* create a new a hbox which includes a image and a label, then return the hbox */
GtkWidget *xpm_label_box (gchar *xpm_name, gchar *label_text);

/* create event_box */
GtkWidget *create_event_box (GtkWidget *vbox, gchar *xpm_name,
                             gchar *label_text);

/* create window's left menus */
void create_menu_actions (void);

/* set the menu visiable */
void set_action_property (GtkAction *action, gchar *menu_action_text,
                          gboolean action_flag);

/* add folders or sms to tree_view */
void list_store_append (GtkListStore *store, GtkWidget *check_button,
                        GdkPixbuf *icon, gint id, gchar *name);

/* create the column */
GtkTreeViewColumn *create_col (GtkCellRenderer *renderer, gchar *rendererType,
                               guint col_num);

/* create tree_view */
GtkWidget *create_tree_view (GtkListStore *store);

gboolean view_msg_selection_func (GtkTreeSelection *selection,
                                  GtkTreeModel *model,
                                  GtkTreePath *path,
                                  gboolean path_currently_selected,
                                  gpointer userdata);

/* the function about selection row of tree_view,get the msg_id or folder_id */
gboolean view_folder_selection_func (GtkTreeSelection *selection, 
                                     GtkTreeModel *model,
                                     GtkTreePath *path, 
                                     gboolean path_currently_selected,
                                     gpointer userdata);

/* the function about selection row of tree_view,get the msg_id or folder_id */
gboolean view_sms_selection_func (GtkTreeSelection *selection, 
                                  GtkTreeModel *model,
                                  GtkTreePath *path,
                                  gboolean path_currently_selected,
                                  gpointer userdata);

/* when create sms, change the contact tree_view cursor */
gboolean contact_changed (GtkTextView *textview, GdkEventKey *event,
                          GtkWidget *textview_content);

/* when create sms, change the content tree_view cursor */
gboolean content_changed (GtkTextView *textview, GdkEventKey *event,
                          GtkWidget *textview_contact);

/* set the menu's lable in window */
gboolean setup_softkeys (GtkWidget *window, gchar *menu_left_str,
                         gchar *menu_right_str);

/* destory window with right menu */
gboolean destroy_window (GtkWidget *widget, gint key_number, gpointer data);

/* get the sms list from the folder */
gboolean get_smses_from_folder (GtkListStore *store, gint fid);

/* get the folders about sms */
void get_folders (GtkListStore *store);

/* set the cursor on the tree_view path */
void set_tree_view_cursor (GtkTreeView *treeview, gchar *path_str);

/* refresh the sms list from the folder */
void refresh_smses_from_folder (GtkListStore *store, gint fid);

/* create the pop window */
GtkWidget *create_pop_window (gchar *pix_name, gchar *text);

/* event action about the left menu */
gboolean popup_key_press_handler (GtkWidget *widget, GdkEventKey *event,
                                  gpointer user_data);

/* when left menu popdown, menus change */
void popup_mapping_handler (GtkWidget *widget, gpointer user_data);

/* create sms, the show menus change */
gboolean menu_left_change_to_phonebook (GtkWidget *widget, GdkEventFocus *event,
                                        gpointer user_data);

/* create the menus */
void menus_create (GtkWidget *window, gchar *menu_left, gchar *menu_right);

/* create sms, the show menus change */
gboolean menu_left_change_to_options (GtkWidget *widget, GdkEventFocus *event,
                                      gpointer user_data);

/* set the scroll window property */
void set_sw_scroll_bar_property (GtkWidget *sw, GtkWidget *widget);

/* create gpe window */
GtkWidget *create_gpe_window (gchar *menu_left, gchar *menu_right,
                              gchar *menu_left_str,
                              gchar *menu_right_str);

/* global event function about gpe_window */
void softkey_activated (GtkWidget *widget, gint key_number, gpointer data);

/* refresh the inbox folder */
gboolean reload_inbox ();

/* close the phone service */
void service_close ();


GtkWidget *create_waiting_window (gchar *pix_name, gchar *text);

/* free list */
void msg_sms_free_list (GList *list);

/* app msg quit */
void app_msg_quit (void);

/* get icon with sms status */
void get_icon_by_status (GdkPixbuf **icon, msg_status_t status);

//show the name from abook */
void get_contacts_from_addrbook (void);

/* search name and photo path with number */
gboolean search_from_addressbook (gint index, const gchar *num,
                                  gchar **name_b,
                                  gchar **photo_path_b);

/* get the current window */
GtkWidget *get_current_window (void);

/* destroy the current window */
void destroy_current_window (GtkWidget *window);


void msg_sms_free_hashtable (GHashTable *hash_table);

gboolean choose_msg_with_OK (GtkWidget *widget, GdkEventKey *event,
                             gpointer user_data);

gboolean tree_view_key_press_handler (GtkWidget *widget, GdkEventKey *event,
                                      gpointer user_data);

#endif /* __MSG_SMS_CALLBACKS_H__ */

