/*  sms - application for sending, receiving and management of short message
 *
 *  Authors: YIN Huasong, WU Yu <yu.wu@orange-ftgroup.com> 
 *  
 *  This software and associated documentation files (the "Software") 
 *  are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *  All Rights Reserved. 
 *
 *  A copyright license is hereby granted for redistribution and use of 
 *  the Software in source and binary forms, with or without modification, 
 *  provided that the following conditions are met: 
 *  - Redistributions of source code must retain the above copyright notice, 
 *  this copyright license and the following disclaimer. 
 *  - Redistributions in binary form must reproduce the above copyright 
 *   notice, this copyright license and the following disclaimer in the 
 *  documentation and/or other materials provided with the distribution. 
 *  - Neither the name of LiPS  nor the names of its Members may be used 
 *  to endorse or promote products derived from the Software without 
 *  specific prior written permission. 
 *
 *  A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *  to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *  distribute any implementation compliant with the any specification adopted 
 *  by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *  and non-discriminatory terms and conditions and under reciprocity, as 
 *  regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *  "AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *  AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *  ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *  POSSIBILITY OF SUCH DAMAGE. 
 */




#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <unistd.h>
#include <fcntl.h>
#include <gtk/gtk.h>
#include <gpe/gpeskwindow.h>
/* gpe includes */
#include <gpe/init.h>           /* gpe_application_init */
#include <gpe/pixmaps.h>        /* gpe_load_icons */
#include <gpe/spacing.h>        /* gpe_get_border, gpe_get_boxspacing */


#include <sys/stat.h>
#include <time.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include <glib.h>
#include <gdk/gdkkeysyms.h>

#include <message.h>
#include <pim.h>
#include <iac.h>

#define LOCK_FILE "/tmp/.sms.lock"

#include "callbacks.h"
#include "sms-iac.h"
#include "sms-app.h"


extern GHashTable *hashtable;   /* save every window's handle */
extern gint array[10];          /* use in hashtable */
extern gint count;              /* use in hashtable */

extern gint time_out_id;        /* timer */
extern msg_sid_t sid;           /* session id */
extern gint service_start;      /* init service */ 


msg_sms_create_text_view_t textview;
msg_sms_tree_col_t tree_col;
msg_sms_test_t msgtest;
msg_sms_flag_t sms_flag;
msg_sms_tree_view_t sms_treeview;
msg_sms_tree_view_path_t sms_path;

msg_msgid_t send_msgid;
guint ps_timeout_id;

int phone_server_ready;

GtkUIManager *ui_manager;

GMainLoop *loop;                 /* session */

GtkWidget *popup_window;

GtkWidget *sending_window;

GList *contact_from_addrbook;
gint pre_list_count = 0;


gchar *folder_menu_action_texts[15] = { "/ui/popupFolderOptions-left/saveContact",    /* 0  */
                                        "/ui/popupFolderOptions-left/copyAdd",        /* 1  */
                                        "/ui/popupFolderOptions-left/reply",          /* 2  */
                                        "/ui/popupFolderOptions-left/callBack",       /* 3  */
                                        "/ui/popupFolderOptions-left/edit",           /* 4  */
                                        "/ui/popupFolderOptions-left/recycle",        /* 5  */
                                        "/ui/popupFolderOptions-left/cleanTrash",     /* 6  */
                                        "/ui/popupFolderOptions-left/sort",           /* 7  */
                                        "/ui/popupFolderOptions-left/search",         /* 8  */
                                        "/ui/popupFolderOptions-left/multiSel",       /* 9  */
                                        "/ui/popupFolderOptions-left/view",           /* 10 */
                                        "/ui/popupFolderOptions-left/sendFolder",     /* 11 */
                                        "/ui/popupFolderOptions-left/forward",        /* 12 */
                                        "/ui/popupFolderOptions-left/delete",         /* 13 */
                                        "/ui/popupFolderOptions-left/moveTo"          /* 14 */
};
gchar *view_menu_action_texts[11] = { "/ui/popupViewOptions-left/saveContact",        /* 0  */
                                      "/ui/popupViewOptions-left/copyAdd",            /* 1  */
                                      "/ui/popupViewOptions-left/reply",              /* 2  */
                                      "/ui/popupViewOptions-left/callBack",           /* 3  */
                                      "/ui/popupViewOptions-left/edit",               /* 4  */
                                      "/ui/popupViewOptions-left/recycle",            /* 5  */
                                      "/ui/popupViewOptions-left/cleanTrash",         /* 6  */
                                      "/ui/popupViewOptions-left/sendFolder",         /* 7  */
                                      "/ui/popupViewOptions-left/forward",            /* 8  */
                                      "/ui/popupViewOptions-left/delete",             /* 9  */
                                      "/ui/popupViewOptions-left/moveTo"              /* 10 */
};

static gint make_sms_window (void);             /* create sms window */
static GtkWidget *make_msg_window (void);       /* main window of msg */
static void make_mms_window (GtkWidget *);      /* create mms window */
static void make_sms_write (gchar *, gchar *);  /* the UI after importing addressbook */
static GtkWidget *make_sms_sending (void);
static void make_sms_saving (void);             /* no use,because of using dialog */
static gint make_sms_view_test (GtkTreeView *treeview, GtkTreePath *path, GtkTreeViewColumn *column, gpointer data);  /* view 0704 sms */
static void make_sms_delete ();                 /* no use,because of using dialog */
static void make_sms_moving (GtkWidget *);      /* no use,because of using dialog */
static void make_save_contact (GtkWidget *);    /* only define */
static void make_copy_add (GtkWidget *);        /* only define */

static void make_sms_folder (gint sms_folder_id);       /* create UI of folders about sms */

static void make_sms_null_contact_dialog (void);

static void make_sms_setting (void);
static void msg_sms_new_status (void);
static gboolean close_sms_create_window (void);

void cancel (GtkWidget *widget);

gboolean
is_phoneserver_ready ()
{
  int fd = 0;
  struct flock flock;

  if (phone_server_ready)
    return TRUE;

  if (g_file_test (LOCK_FILE, G_FILE_TEST_EXISTS))
  {

    g_print ("%s(): lock file exists\n", __FUNCTION__);
    fd = open (LOCK_FILE, O_WRONLY);

    flock.l_type = F_WRLCK;
    flock.l_whence = SEEK_SET;
    flock.l_start = 0;
    flock.l_len = 0;

    fcntl (fd, F_GETLK, &flock);

    close (fd);

    if (flock.l_type == F_UNLCK)
    {
      g_print ("%s(): file is unlocked\n", __FUNCTION__);
      phone_server_ready = 1;
      return TRUE;
    }
  }

  return FALSE;
}

static void
make_sms_setting (void)
{
}
void
make_sms_pre_write ()
{
  make_sms_write (NULL, NULL);
}

void
msg_sms_new_incoming (void)
{
  msg_sms_new_status ();
}

static void
msg_sms_new_status (void)
{
  if (count == MSG_SMS_FOLDERS || count == MSG_SMS_CREATE_OR_FOLDERLIST)
  {
    reload_inbox ();
  }
  if (count > MSG_SMS_CREATE_OR_FOLDERLIST)
  {
    if (!(sms_flag.view_new_flag))
      sms_flag.view_new_flag = TRUE;
  }
}

static void
msg_sms_remove (GtkTreeView *view, gchar *path_str, GtkTreeViewColumn *col,
                gpointer user_data)
{
  GtkTreeModel *model;
  GtkTreeIter iter;
  GtkTreePath *path = NULL;

  g_print ("Row has been double-clicked. Removing row.\n");

  path = gtk_tree_path_new_from_string (path_str);
  model = gtk_tree_view_get_model (view);

  if (!gtk_tree_model_get_iter (model, &iter, path))
    return;                                             /* path describes a non-existing row - should not happen */

  gtk_list_store_remove (GTK_LIST_STORE (model), &iter);
}


static gboolean
sms_delete (GtkWidget *dialog)
{
  GtkWidget *window = NULL;

  g_source_remove (time_out_id);

  if (msgtest.garray != NULL)
  {
    gint i;

    for (i = 0; i < msgtest.garray->len; i++)
    {
      if (msg_folder_remove_msg (sid, msgtest.fid,
                                 g_array_index (msgtest.garray, gint, i)) == 0)
        g_print ("delete OK!\n");
    }
    g_array_free (msgtest.garray, TRUE);
    msgtest.garray = NULL;
  }
  else
  {
    if (msg_folder_remove_msg (sid, msgtest.fid, msgtest.msgid) == 0)
      g_print ("test\n");
  }

  if (count > MSG_SMS_CREATE_OR_FOLDERLIST)
  {
    if (msgtest.garray != NULL)
      msgtest.garray = NULL;
    destroy_current_window (get_current_window ());

  }
  window = get_current_window ();
  gpe_skwindow_set_softkey_label (GPE_SKWINDOW (window), SK_LEFT, "Options");
  refresh_smses_from_folder (sms_treeview.sms_list_store, msgtest.fid);

  sleep (1);

  gtk_widget_destroy (dialog);

  gtk_window_present (GTK_WINDOW (window));

  sms_path.sms_number = sms_path.sms_number - 1;

  if (sms_flag.multi_select_flag)
    gtk_widget_grab_focus (GTK_WIDGET (sms_treeview.sms_list_tree_view));
  else
  {
    if (atoi (sms_path.sms_list_path) < sms_path.sms_number)
      set_tree_view_cursor (sms_treeview.sms_list_tree_view,
                            sms_path.sms_list_path);
    else
    {
      gchar *path_str = NULL;
      gint num_path;

      num_path = sms_path.sms_number - 1;
      path_str = g_strdup_printf ("%d", num_path);
      set_tree_view_cursor (sms_treeview.sms_list_tree_view, path_str);
      g_free (path_str);
    }
  }

  return FALSE;
}


void
send_success_cb (msg_sid_t sid, msg_evt_t *evt)
{

  msg_msgid_t id = *(msg_msgid_t *) evt->evt_data;

  msg_update_status (sid, id, MSG_SENT);
  msg_folder_move_msg (sid, MSG_FOLDER_OUTBOX, MSG_FOLDER_SENT, id);

  if (!sending_window)
  {
    return;
  }

  if (id != send_msgid)
  {
    return;
  }
}


void
send_failed_cb (msg_sid_t sid, msg_evt_t *evt)
{
  g_print ("msg %d is send failed\n", *(uint32_t *) evt->evt_data);
  GtkWidget *notice_window = NULL;
  char text[64];

  msg_msgid_t id = *(msg_msgid_t *) evt->evt_data;

  msg_update_status (sid, id, MSG_SENT_FAILED);

  g_print ("**********************************failed id = %d\n", id);

  memset (text, 0x00, sizeof (text));
  
  sprintf (text, "Failed To Send SMS");

  notice_window = create_pop_window ("pop_icon_notice.png", text);

  g_signal_connect_swapped (G_OBJECT (notice_window), "softkey_activated",
                            G_CALLBACK (destroy_window), notice_window);

  gtk_widget_show_all (notice_window);
}


void
msg_incoming_cb (msg_sid_t sid, msg_evt_t *evt)
{

  g_print ("msg_incoming_cb: msg %d is recieved\n",
           *(uint32_t *) evt->evt_data);

  GtkWidget *notice_window = NULL;

  notice_window = create_pop_window ("pop_icon_info.png", "New Message...");

  g_signal_connect_swapped (G_OBJECT (notice_window), "softkey_activated",
                            G_CALLBACK (destroy_window), notice_window);

  gtk_widget_show_all (notice_window);

  if (count == MSG_SMS_FOLDERS || count == MSG_SMS_CREATE_OR_FOLDERLIST)
    reload_inbox ();
  if (count > MSG_SMS_CREATE_OR_FOLDERLIST)
    sms_flag.view_new_flag = TRUE;
}


void
status_report_cb (msg_sid_t sid, msg_evt_t *evt)
{
  msg_message_t *message = NULL;
  msg_msgid_t msgid = *(msg_msgid_t *) evt->evt_data;
  GtkWidget *notice_window = NULL;
  char text[64];
  char *name = NULL;

  g_print ("msg %d reached destination\n", msgid);

  message = msg_test_get_msg_by_id (sid, msgid);

  memset (text, 0x00, sizeof (text));

  search_from_addressbook (0, msg_get_recipientlist (sid, message), &name,
                           NULL);

  if (name)
    sprintf (text, "Message Reached:\n%s", name);
  else
    sprintf (text, "Message Reached:\n%s",
             msg_get_recipientlist (sid, message));

  notice_window = create_pop_window ("pop_icon_info.png", text);

  time_out_id = g_timeout_add (1000, (GtkFunction) destroy_window, notice_window);      /* remove time_out_id */

  gtk_widget_show_all (notice_window);

  g_free (name);

  msg_release (sid, message);

}

static GtkWidget *
make_msg_window (void)
{
  GtkWidget *window;
  GtkWidget *sw;
  GtkWidget *vbox;
  GtkWidget *eventbox;
  GtkWidget *tree_view;

  GtkListStore *store;
  GtkTreeSelection *selection;

  window = gpe_skwindow_new ();
  g_hash_table_insert (hashtable, &array[count], window);

  /* install global key handler */
  g_signal_connect_swapped (G_OBJECT (window), "softkey_activated",
                            G_CALLBACK (softkey_activated), window);


  /* set up softkey usage */
  setup_softkeys (window, "OK", "Back");

  vbox = gtk_vbox_new (FALSE, 0);
  gtk_container_add (GTK_CONTAINER (window), vbox);

  eventbox = create_event_box (vbox, "icon_reading_msg.png", "MSG");

  sw = make_swindow (vbox);

  store = gtk_list_store_new (COLUMN_NUMS, G_TYPE_BOOLEAN,
                              GDK_TYPE_PIXBUF, G_TYPE_INT, G_TYPE_STRING);
  tree_col.icon = get_icon ("icon_reading_msg.png");
  tree_col.id = MSG_FOLDER_SMS_ROOT;
  strcpy (tree_col.name, "SMS");
  list_store_append (GTK_LIST_STORE (store), NULL, tree_col.icon, tree_col.id,
                     tree_col.name);
  tree_col.id = MSG_FOLDER_MMS_ROOT;
  strcpy (tree_col.name, "MMS");
  list_store_append (store, NULL, tree_col.icon, tree_col.id, tree_col.name);

  tree_view = create_tree_view (store);
  sms_treeview.msg_list_tree_view = GTK_TREE_VIEW (tree_view);

  selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (tree_view));
  gtk_tree_selection_set_select_function (selection, view_msg_selection_func,
                                          NULL, NULL);
  g_signal_connect (tree_view, "row-activated", (GCallback) choose_msg, NULL);
  set_sw_scroll_bar_property (sw, tree_view);
  gtk_widget_show_all (window);

  return window;

}

void
choose_msg (GtkTreeView *treeview, GtkTreePath *path,
            GtkTreeViewColumn *column, gpointer data)
{
  switch (msgtest.root_fid)
  {
    case MSG_FOLDER_SMS_ROOT:
      make_sms_window ();
      break;
    case MSG_FOLDER_MMS_ROOT:
      make_mms_window (NULL);
      break;
    default:
      break;
  }
}

static gint
make_sms_window ()
{
  GtkWidget *window, *sw;
  GtkWidget *vbox;
  GtkWidget *eventbox;
  GtkWidget *tree_view;
  GtkListStore *store;
  GtkTreeSelection *selection;


  /* create application window */
  window = create_gpe_window ("", "", "OK", "Back");

  count++;
  g_hash_table_insert (hashtable, &array[count], window);

  vbox = gtk_vbox_new (FALSE, 0);
  gtk_container_add (GTK_CONTAINER (window), vbox);

  eventbox = create_event_box (vbox, "icon_reading_msg.png", "SMS");

  sw = make_swindow (vbox);

  store = gtk_list_store_new (COLUMN_NUMS, G_TYPE_BOOLEAN,
                              GDK_TYPE_PIXBUF, G_TYPE_INT, G_TYPE_STRING);
  sms_treeview.folder_list_store = store;
  get_folders (store);

  tree_view = create_tree_view (store);

  sms_treeview.folder_list_tree_view = GTK_TREE_VIEW (tree_view);

  selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (tree_view));
  gtk_tree_selection_set_select_function (selection, view_folder_selection_func,
                                          NULL, NULL);
  g_signal_connect (tree_view, "row-activated", (GCallback) choose_sms, NULL);

  set_sw_scroll_bar_property (sw, tree_view);

  gtk_widget_show_all (window);
  return 0;
}

static gboolean
destroy_waiting_window (GtkWidget *dialog)
{

  g_print ("***********************destroy waiting....sms_number %d\n",
           sms_path.sms_number);
  
  if (sms_flag.open_flag)
  {
    g_source_remove (time_out_id);
    time_out_id = g_timeout_add (1, (GtkFunction) destroy_window, dialog);
    sms_flag.open_flag = TRUE;
    return FALSE;
  }
  else
    return TRUE;
}

void
show_waiting_window (void)
{
  GtkWidget *dialog = NULL;

  dialog = create_pop_window ("pop_icon_info.png", "Please Waiting...");

  time_out_id = g_timeout_add (1, (GtkFunction) destroy_waiting_window, dialog);

  gtk_widget_show_all (dialog);

  return;
}

gboolean
destroy_wait_ps_dialog (GtkWidget *widget, gint key_number, gpointer data)
{
  if (key_number == SK_RIGHT)
  {
    if (ps_timeout_id)
    {
      g_source_remove (ps_timeout_id);
      ps_timeout_id = 0;
    }

    gtk_widget_destroy (widget);
  }

  return FALSE;
}

gboolean destory_wait_phoneserver_dialog (GtkWidget *dialog);

void
wait_phone_server ()
{

  GtkWidget *dialog = NULL;

  dialog = create_pop_window ("pop_icon_info.png", "Please Waiting....");
  gtk_window_set_default_size (GTK_WINDOW (dialog), 240, 80);
  gtk_window_move (GTK_WINDOW (dialog), 0, 120);

  g_signal_connect_swapped (G_OBJECT (dialog), "softkey_activated",
                            G_CALLBACK (destroy_wait_ps_dialog), dialog);
  gtk_widget_show_all (dialog);


  if (dialog)
    ps_timeout_id =
      g_timeout_add (500, (GSourceFunc) destory_wait_phoneserver_dialog,
                     dialog);

  return;
}

void
choose_sms (GtkTreeView *treeview, GtkTreePath *path,
            GtkTreeViewColumn *column, gpointer data)
{
  GtkWidget *dialog = NULL;
  int fd = 0;
  struct flock flock;


  switch (msgtest.fid)
  {
    case MSG_SMS_CREATE:
      make_sms_write (NULL, NULL);
      break;
    case MSG_SMS_SETTING:
      make_sms_setting ();
      break;
    default:
      if (is_phoneserver_ready ())
        make_sms_folder (msgtest.fid);
      else
        wait_phone_server ();
      break;
  }
}

static void
make_mms_window (GtkWidget *widget)
{
}

static void
make_sms_write (gchar *contact, gchar *body)
{

  GtkWidget *window;
  GtkWidget *vbox;
  GtkWidget *hbox;
  GtkWidget *textview_contact;
  GtkWidget *textview_content;
  GtkWidget *eventbox;
  GtkTextBuffer *buffer;
  gchar contact_text[60];

  strcpy (contact_text, "To:");
  if (contact != NULL)
    strcat (contact_text, contact);
  
  /* create application window */
  window = create_gpe_window ("", "", "PhoneBook", "Back");

  sms_flag.create_flag = TRUE;

  if (!sms_flag.dbus_invoke_flag)
  {
    count++;
    g_hash_table_insert (hashtable, &array[count], window);
  }

  vbox = gtk_vbox_new (FALSE, 0);
  gtk_container_add (GTK_CONTAINER (window), vbox);
  eventbox = create_event_box (vbox, "icon_create.png", "Text message");

  hbox = gtk_hbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, TRUE, 2);
  gtk_widget_set_size_request (hbox, 240, 40);

  textview_contact = gtk_text_view_new ();
  gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (textview_contact),
                               GTK_WRAP_WORD_CHAR);

  textview_content = gtk_text_view_new ();
  gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (textview_content),
                               GTK_WRAP_WORD_CHAR);

  g_signal_connect (textview_contact, "key-press-event",        
                    G_CALLBACK (contact_changed), textview_content);
  g_signal_connect (textview_contact, "focus-in-event", 
                    G_CALLBACK (menu_left_change_to_phonebook), NULL);

  buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (textview_contact));
  gtk_text_buffer_set_text (buffer, contact_text, -1);
  textview.buffer_contact = buffer;
  gtk_box_pack_start (GTK_BOX (hbox), textview_contact, TRUE, TRUE, 2);

  g_signal_connect (textview_content, "key-press-event",        
                    G_CALLBACK (content_changed), textview_contact);
  g_signal_connect (textview_content, "focus-in-event", 
                    G_CALLBACK (menu_left_change_to_options), NULL);
  buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (textview_content));
  if (body)
    gtk_text_buffer_set_text (buffer, body, -1);
  textview.buffer_content = buffer;
  gtk_box_pack_start (GTK_BOX (vbox), textview_content, TRUE, TRUE, 3);
  if (contact != NULL)
    gtk_widget_grab_focus (textview_content);

  gtk_widget_show_all (window);
}


static gboolean
close_sms_create_window (void)
{
  if (sms_flag.start_from_abook_flag)
  {
    app_msg_quit ();
  }
  else if (sms_flag.dbus_invoke_flag)
  {
    destroy_current_window (get_current_window ());
    sms_flag.dbus_invoke_flag = FALSE;
  }
  else
  {
    destroy_current_window (get_current_window ());

    if (count != MSG_SMS_FOLDERS)
      gpe_skwindow_set_softkey_label (GPE_SKWINDOW (get_current_window ()),
                                      SK_LEFT, "Options");
  }

  sms_flag.create_flag = FALSE;

  msg_sms_free_list (contact_from_addrbook);
  contact_from_addrbook = NULL;

  return FALSE;                 /* time_out */
}


static GtkWidget *
make_sms_sending (void)
{
  GtkWidget *window =
    create_waiting_window ("pop_icon_waiting.png", "Sending Message");

  gtk_widget_show_all (window);

  return window;

}

static void
make_sms_null_contact_dialog (void)
{
  GtkWidget *dialog = NULL;

  dialog = create_pop_window ("pop_icon_info.png", "Please Input contact...");

  g_print ("***********************************Please Input contact...\n");

  gtk_widget_show_all (dialog);

  time_out_id = g_timeout_add (2000, (GtkFunction) destroy_window, dialog);

}


static void
make_sms_saving (void)
{
  GtkWidget *dialog = NULL;

  dialog = create_pop_window ("pop_icon_info.png", "Saving To Drafts...");

  time_out_id = g_timeout_add (1800, (GtkFunction) destroy_window, dialog);

  gtk_widget_show_all (dialog);
}


static void
make_sms_folder (gint fid)
{
  GtkWidget *window;
  GtkWidget *vbox;
  GtkWidget *sw;
  GtkWidget *eventbox;
  GtkAction *action;
  GtkWidget *treeview;
  GtkListStore *store;
  GtkTreeSelection *selection;

  gchar *image_text = NULL;
  gchar *label_text = NULL;
  gint i;

  gchar *popupOptions = NULL;

  if (sms_flag.multi_select_flag)
  {
    popupOptions = "/popupMultiOptions-left";
  }
  else
  {
    popupOptions = "/popupFolderOptions-left";
  }

  /* create application window */
  window = create_gpe_window (popupOptions, "", "Options", "Back");

  count++;
  g_hash_table_insert (hashtable, &array[count], window);

  create_menu_actions ();
  for (i = 0; i < 7; i++)
    set_action_property (action, folder_menu_action_texts[i], FALSE);

  switch (fid)
  {
    case MSG_FOLDER_INBOX:
      image_text = "icon_no_new_msg.png";
      label_text = "Inbox";
      set_action_property (action, folder_menu_action_texts[11], FALSE);
      for (i = 2; i < 4; i++)
        set_action_property (action, folder_menu_action_texts[i], TRUE);

      break;
    case MSG_FOLDER_SENT:
      image_text = "icon_sent.png";
      label_text = "Sent";
      break;
    case MSG_FOLDER_OUTBOX:
      image_text = "icon_outbox.png";
      label_text = "Outbox";
      break;
    case MSG_FOLDER_DRAFT:
      image_text = "icon_draft.png";
      label_text = "Draft";
      break;
    case MSG_FOLDER_REPORT:
      image_text = "icon_reports.png";
      label_text = "Report";
      for (i = 10; i < 13; i++)
        set_action_property (action, folder_menu_action_texts[i], FALSE);
      break;
    case MSG_FOLDER_TRASH:
      image_text = "icon_recycle.png";
      label_text = "Trash";
      for (i = 11; i < 15; i++)
        set_action_property (action, folder_menu_action_texts[i], FALSE);
      for (i = 5; i < 7; i++)
        set_action_property (action, folder_menu_action_texts[i], TRUE);
      break;
    case MSG_FOLDER_TEMPLATE:
      image_text = "icon_template.png";
      label_text = "Template";
      set_action_property (action, folder_menu_action_texts[12], FALSE);
      set_action_property (action, folder_menu_action_texts[14], FALSE);
      set_action_property (action, folder_menu_action_texts[4], TRUE);
      break;
    default:
      break;
  }

  vbox = gtk_vbox_new (FALSE, 0);
  gtk_container_add (GTK_CONTAINER (window), vbox);

  eventbox = create_event_box (vbox, image_text, label_text);

  sw = make_swindow (vbox);

  store = gtk_list_store_new (COLUMN_NUMS, G_TYPE_BOOLEAN,
                              GDK_TYPE_PIXBUF, G_TYPE_INT, G_TYPE_STRING);

  get_smses_from_folder (store, fid);

  treeview = create_tree_view (store);

  if (!sms_flag.multi_select_flag)
  {
    sms_treeview.sms_list_store = store;
    sms_treeview.sms_list_tree_view = GTK_TREE_VIEW (treeview);
  }

  selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview));
  gtk_tree_selection_set_select_function (selection, view_sms_selection_func,
                                          NULL, NULL);

  g_signal_connect (treeview, "row-activated", (GCallback) make_sms_view_test,
                    NULL);

  set_sw_scroll_bar_property (sw, treeview);

  g_print ("&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&show the list window\n");

  gtk_widget_show_all (window);

}


int
get_time (time_t timestamp, char *result)
{
  struct tm time;
  char format[24];

  memset (format, 0x00, sizeof (format));
  memset (&time, 0x00, sizeof (time));

  localtime_r (&timestamp, &time);

  (time.tm_mon < 10) ? strcpy (format, "0%d/") : strcpy (format, "%d/");

  (time.tm_mday < 10) ? strcat (format, "0%d/%d ") : strcat (format, "%d/%d ");

  (time.tm_hour < 10) ? strcat (format, "0%d:") : strcat (format, "%d:");

  (time.tm_min < 10) ? strcat (format, "0%d") : strcat (format, "%d");


  sprintf (result, format, time.tm_mon + 1, time.tm_mday, time.tm_year + 1900,
           time.tm_hour, time.tm_min);

  return 0;
}



static gint
make_sms_view_test (GtkTreeView *treeview, GtkTreePath *path,
                    GtkTreeViewColumn *column, gpointer data)
{
  GtkWidget *window;
  GtkWidget *vbox;
  GtkWidget *text;
  GtkWidget *eventbox;
  GtkTextBuffer *buffer;
  GtkAction *action;

  gint size = 0;
  gchar *body = NULL;
  const gchar *contact = NULL;
  gchar contact_text[60];
  msg_message_t *msg = NULL;

  msg_time_t msg_time;
  char time_text[64];
  gchar *name = NULL;

  gchar *image_text = NULL;
  gchar *label_text = NULL;


  memset (time_text, 0x00, sizeof (time_text));

  gint i;
  msg = msg_test_get_msg_by_id (sid, msgtest.msgid);
  
  /* create application window */
  window = create_gpe_window ("/popupViewOptions-left", "", "Options", "Back");

  count++;
  g_hash_table_insert (hashtable, &array[count], window);

  create_menu_actions ();
  for (i = 0; i < 7; i++)
    set_action_property (action, view_menu_action_texts[i], FALSE);
  switch (msgtest.fid)
  {
    case MSG_FOLDER_INBOX:
      image_text = "icon_reading_msg.png";
      label_text = "View_Inbox";
      for (i = 0; i < 4; i++)
        set_action_property (action, view_menu_action_texts[i], TRUE);
      set_action_property (action, view_menu_action_texts[7], FALSE);
      break;
    case MSG_FOLDER_SENT:
      image_text = "icon_reading_msg.png";
      label_text = "View_Sent";
      break;
    case MSG_FOLDER_OUTBOX:
      image_text = "icon_reading_msg.png";
      label_text = "View_Outbox";
      break;
    case MSG_FOLDER_DRAFT:
      image_text = "icon_reading_msg.png";
      label_text = "View_Drafts";
      break;
    case MSG_FOLDER_TRASH:
      image_text = "icon_reading_msg.png";
      label_text = "View_Trash";
      for (i = 5; i < 7; i++)
        set_action_property (action, view_menu_action_texts[i], TRUE);
      for (i = 7; i < 11; i++)
        set_action_property (action, view_menu_action_texts[i], FALSE);
      break;
    case MSG_FOLDER_TEMPLATE:
      image_text = "icon_reading_msg.png";
      label_text = "Template";
      set_action_property (action, view_menu_action_texts[4], TRUE);
      break;
    default:
      break;
  }

  vbox = gtk_vbox_new (FALSE, 0);
  gtk_container_add (GTK_CONTAINER (window), vbox);
  eventbox = create_event_box (vbox, image_text, label_text);

  text = gtk_text_view_new ();
  gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (text), GTK_WRAP_WORD_CHAR);
  gtk_widget_set_size_request (text, 240, 40);
  buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (text));
  gtk_text_view_set_editable (GTK_TEXT_VIEW (text), FALSE);
  gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (text), FALSE);
  msg_time = msg_get_timestamp (sid, msg);
  get_time (msg_time, time_text);

  if (msgtest.fid == MSG_FOLDER_INBOX)
  {
    strcpy (contact_text, "From: ");
    contact = msg_get_from (sid, msg);

    if (msg_get_status (sid, msg) == MSG_RECEIVED_UNREAD)
    {
      msg_update_status (sid, msgtest.msgid, MSG_RECEIVED_READ);        
      sms_flag.view_new_flag = TRUE;
    }
  }
  else
  {
    strcpy (contact_text, "To: ");
    contact = msg_get_recipientlist (sid, msg);
  }
  if (search_from_addressbook (0, contact, &name, NULL))
  {
    if (name)
      contact = name;
  }

  strcat (contact_text, contact);
  strcat (contact_text, "\nTime: ");
  strcat (contact_text, time_text);

  gtk_text_buffer_set_text (GTK_TEXT_BUFFER (buffer), contact_text, -1);
  gtk_box_pack_start (GTK_BOX (vbox), text, FALSE, TRUE, 2);

  text = gtk_text_view_new ();
  gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (text), GTK_WRAP_WORD_CHAR);
  buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (text));
  gtk_text_view_set_editable (GTK_TEXT_VIEW (text), FALSE);
  gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (text), FALSE);
  body = msg_get_body_by_id (sid, msgtest.msgid, &size);

  gtk_text_buffer_set_text (GTK_TEXT_BUFFER (buffer), body, -1);
  gtk_box_pack_start (GTK_BOX (vbox), text, TRUE, TRUE, 3);

  gtk_widget_show_all (window);

  msg_release (sid, msg);
  g_free (body);
  g_free (name);
  return 0;

}



static void
make_sms_delete ()
{
  GtkWidget *dialog;

  dialog = create_pop_window ("pop_icon_info.png", "MSG Deleting...");

  time_out_id = g_timeout_add (1000, (GSourceFunc) sms_delete, dialog);

  gtk_widget_show_all (dialog);
}

static void
make_sms_moving (GtkWidget *widget)
{
  GtkWidget *dialog;

  dialog = create_pop_window ("pop_icon_info.png", "Moving To Drafts...");

  gtk_widget_show_all (dialog);

  sleep (2);

  gtk_widget_destroy (dialog);

}


static void
make_save_contact (GtkWidget *widget)
{
}

static void
make_copy_add (GtkWidget *window)
{
}

gboolean
sending_window_destroy (GtkWidget *window)
{
  if (!sending_window)
    return FALSE;

  gtk_widget_destroy (sending_window);
  sending_window = NULL;

  g_print ("destroy window\n");

  if (sms_flag.start_from_abook_flag)
  {
    app_msg_quit ();
  }
  else if (sms_flag.dbus_invoke_flag)
  {
    destroy_current_window (get_current_window ());
    sms_flag.dbus_invoke_flag = FALSE;
  }
  else
  {
    destroy_current_window (get_current_window ());

    if (count != MSG_SMS_FOLDERS)
      gpe_skwindow_set_softkey_label (GPE_SKWINDOW (get_current_window ()),
                                      SK_LEFT, "Options");

    if (msgtest.fid == MSG_FOLDER_INBOX && sms_flag.view_new_flag)
    {
      refresh_smses_from_folder (sms_treeview.sms_list_store, msgtest.fid);
      set_tree_view_cursor (sms_treeview.sms_list_tree_view,
                            sms_path.sms_list_path);
    }
  }

  sms_flag.create_flag = FALSE;

  msg_sms_free_list (contact_from_addrbook);
  contact_from_addrbook = NULL;

  return FALSE;

}

typedef struct
{
  gchar **contact_a;
  gchar *text_content;
} send_trans_t;

gpointer
send_thread (gpointer arg)
{
  char *contact = NULL;
  msg_message_t *msg = NULL;
  send_trans_t *trans = (send_trans_t *) arg;
  msg_msgid_t msg_id = 0;
  char **contact_a = trans->contact_a;
  char *text_content = trans->text_content;
  gint i = 0;
  guint j = 0;

  for (i = 0; contact_a[i]; i++)
  {
    if (strcmp (contact_a[i], "") == 0)
      continue;

    msg = msg_new (sid, MSG_SMS);

    j = 0;
    while (j < g_list_length (contact_from_addrbook))
    {

      if (strcmp (contact_a[i], g_list_nth_data (contact_from_addrbook, j)) 
          == 0)
      {
        contact =
          g_strdup_printf ("%s",
                           (gchar *) g_list_nth_data (contact_from_addrbook,
                                                      j + 1));
        g_free (g_list_nth_data (contact_from_addrbook, j));
        g_free (g_list_nth_data (contact_from_addrbook, j + 1));
        contact_from_addrbook = g_list_remove (contact_from_addrbook,
                                               g_list_nth_data
                                               (contact_from_addrbook, j));
        contact_from_addrbook =
          g_list_remove (contact_from_addrbook,
                         g_list_nth_data (contact_from_addrbook, j));

        pre_list_count = g_list_length (contact_from_addrbook);
        break;
      }
      j = j + 2;
    }

    if (!contact)
      contact = g_strdup (contact_a[i]);


    msg_set_recipientlist (sid, msg, contact);
    msg_set_body (sid, msg, text_content, strlen (text_content) + 1);

    if (msg_send (sid, msg) != 0)
      printf ("Send failed\n");
    else
      printf ("Message send\n");

    msg_id = msg_get_id (sid, msg);

    msg_release (sid, msg);
    g_free (contact);
    contact = NULL;
  }

  send_msgid = msg_id;

  g_free (text_content);


  g_strfreev (contact_a);

  g_free (trans);



  if (sms_flag.dbus_invoke_flag)
    sms_flag.dbus_invoke_flag = FALSE;

  return NULL;
}


/* send sms */
static gint
send ()
{
  GtkTextIter start;
  GtkTextIter end;
  GtkTextBuffer *buffer_contact = NULL;
  GtkTextBuffer *buffer_content = NULL;
  msg_message_t *msg = NULL;
  gchar **contact_a = NULL;

  int i = 0;
  int num = 0;
  gint ret = 0;
  send_trans_t *trans;

  gchar *text_contacts;
  gchar *text_content;
  gchar *contacts = NULL;
  gchar *contact = NULL;        

  buffer_contact = textview.buffer_contact;
  buffer_content = textview.buffer_content;

  /* Get iters at the beginning and end of the buffer. */
  gtk_text_buffer_get_bounds (buffer_contact, &start, &end);
  
  /* Retrieve the text. */
  text_contacts =
    gtk_text_buffer_get_text (buffer_contact, &start, &end, FALSE);

  /* Get iters at the beginning and end of the buffer. */
  gtk_text_buffer_get_bounds (buffer_content, &start, &end);
  
  /* Retrieve the text. */
  text_content = gtk_text_buffer_get_text (buffer_content, &start, &end, FALSE);

  contacts = g_strdup (text_contacts + strlen ("To:"));

  if ((g_str_has_suffix (contacts, ";")) || (g_str_has_suffix (contacts, " ")))
     *(contacts + strlen (contacts) - 1) = 0;

  if (contacts)
    g_print ("contacts = %s\n", contacts);

  if (strcmp (contacts, " ") > 0)
  {
    contact_a = g_strsplit_set (contacts, "; ", -1);
    for (i = 0; contact_a[i]; i++)
      num++;

    trans = g_new0 (send_trans_t, 1);
    trans->contact_a = contact_a;
    trans->text_content = text_content;
    sending_window = make_sms_sending ();
    g_timeout_add (5000 * num, (GSourceFunc) sending_window_destroy, NULL);
    g_thread_create (send_thread, trans, FALSE, NULL);
    ret = 0;
  }
  else
  {
    make_sms_null_contact_dialog ();
    ret = -1;
  }
  g_free (text_contacts);

  g_free (contacts);
  return ret;

}

/* save sms */
static gint
save ()
{
  GtkTextIter start;
  GtkTextIter end;
  GtkTextBuffer *buffer_contact = NULL;
  GtkTextBuffer *buffer_content = NULL;
  msg_message_t *msg = NULL;
  gchar **contact_a = NULL;
  msg_msgid_t msg_id = 0;
  int i = 0;

  gchar *text_contacts;
  gchar *text_content;
  gchar *contacts = NULL;
  gchar *contact = NULL;        

  buffer_contact = textview.buffer_contact;
  buffer_content = textview.buffer_content;

  /* Get iters at the beginning and end of the buffer. */
  gtk_text_buffer_get_bounds (buffer_contact, &start, &end);
  
  /* Retrieve the text. */
  text_contacts =
    gtk_text_buffer_get_text (buffer_contact, &start, &end, FALSE);

  /* Get iters at the beginning and end of the buffer. */
  gtk_text_buffer_get_bounds (buffer_content, &start, &end);
  
  /* Retrieve the text. */
  text_content = gtk_text_buffer_get_text (buffer_content, &start, &end, FALSE);

  contacts = g_strdup (text_contacts + strlen ("To:"));

  if ((g_str_has_suffix (contacts, ";")) || (g_str_has_suffix (contacts, " ")))
    *(contacts + strlen (contacts) - 1) = 0;
  if (contacts)
    g_print ("contacts = %s\n", contacts);

  contact_a = g_strsplit_set (contacts, "; ", -1);

  if (g_strv_length (contact_a) > 0)
  {
    make_sms_saving ();

    guint j;

    for (i = 0; contact_a[i]; i++)
    {
      if (strcmp (contact_a[i], "") == 0)
        continue;
      msg = msg_new (sid, MSG_SMS);
      contact = contact_a[i];

      j = 0;
      while (j < g_list_length (contact_from_addrbook))
      {
        if (strcmp (contact_a[i], g_list_nth_data (contact_from_addrbook, j))
            == 0)
        {
          contact =
            g_strdup_printf ("%s",
                             (gchar *) g_list_nth_data (contact_from_addrbook,
                                                        j + 1));
          g_free (g_list_nth_data (contact_from_addrbook, j));
          g_free (g_list_nth_data (contact_from_addrbook, j + 1));
          contact_from_addrbook = g_list_remove (contact_from_addrbook,
                                                 g_list_nth_data
                                                 (contact_from_addrbook, j));
          contact_from_addrbook =
            g_list_remove (contact_from_addrbook,
                           g_list_nth_data (contact_from_addrbook, j));
          pre_list_count = g_list_length (contact_from_addrbook);

          break;
        }
        j = j + 2;
      }

      if (contact)
        g_print ("Test Contact:%s\n", contact);
      msg_set_recipientlist (sid, msg, contact);
      msg_set_body (sid, msg, text_content, strlen (text_content) + 1);

      msg_folder_add_msg (sid, MSG_FOLDER_DRAFT, msg);

      msg_release (sid, msg);
      contact = NULL;

    }

    g_free (text_content);
    g_free (text_contacts);
    g_strfreev (contact_a);
    g_free (contacts);

    g_timeout_add (2000, (GtkFunction) close_sms_create_window, NULL);   /* close_sms_create_window (); */
    return 0;
  }
  else
  {
    make_sms_null_contact_dialog ();
    return -1;
  }
}

static void
insert ()
{
}

static void
get_addrbook ()
{
}

static void
get_template ()
{
}

static void
clean ()
{
  /* make_sms_write (NULL, NULL); */
}

void
cancel (GtkWidget *widget)
{
  gtk_timeout_remove (time_out_id);
  gtk_widget_destroy (widget);

}
static void
view ()
{
  make_sms_view_test (NULL, NULL, NULL, NULL);
}

/* reply sms */
static gint
reply_sms ()
{
  msg_message_t *msg = NULL;
  const gchar *contact = NULL;
  gchar *name = NULL;
  gchar *name_list = NULL;
  gchar *contact_list = NULL;

  msg = msg_test_get_msg_by_id (sid, msgtest.msgid);
  contact = msg_get_from (sid, msg);

  if (search_from_addressbook (0, contact, &name, NULL))
  {
    if (name)
    {
      name_list = g_strdup_printf ("%s", name);
      contact_from_addrbook = g_list_append (contact_from_addrbook, name_list);
      contact_list = g_strdup_printf ("%s", contact);
      contact_from_addrbook =
        g_list_append (contact_from_addrbook, contact_list);

      pre_list_count = g_list_length (contact_from_addrbook);

      contact = name;
    }
  }

  make_sms_write ((gchar *) contact, NULL);
  msg_release (sid, msg);
  g_free (name);
  return 0;
}

/* forward sms */
static void
forward ()
{
  gchar *body = NULL;
  gint size = 0;
  body = msg_get_body_by_id (sid, msgtest.msgid, &size);
  make_sms_write (NULL, body);
  g_free (body);
}

/* delete sms ,but no UI */
static void
delete_sms ()
{
  make_sms_delete ();
}

static void
search ()
{
  gtk_tree_view_set_search_column (sms_treeview.sms_list_tree_view,
                                   COLUMN_TEXT);
}

static void
sort ()
{
}

static void
sort_by_name ()
{
  GtkTreeSortable *sortable;

  sortable = GTK_TREE_SORTABLE (sms_treeview.sms_list_store);
  gtk_tree_sortable_set_sort_column_id (sortable, COLUMN_TEXT,
                                        GTK_SORT_ASCENDING);

}

static void
sort_by_date ()
{
  GtkWidget *window;

  window = get_current_window ();
  sms_flag.sort_date_flag = TRUE;
  destroy_current_window (window);

  make_sms_folder (msgtest.fid);
  sms_flag.sort_date_flag = FALSE;
}

/* multiselect,change the multiSel_flag = TRUE */
static void
multiSel ()
{
  sms_flag.multi_select_flag = TRUE;
  msgtest.garray = g_array_new (FALSE, FALSE, sizeof (gint));
  make_sms_folder (msgtest.fid);
  sms_flag.multi_select_flag = FALSE;

}
static void
move ()
{
}
static void
move_to_SIM ()
{
  make_sms_moving (NULL);
}
static void
move_to_phone ()
{
  make_sms_moving (NULL);
}
static gint
callBack ()
{
  msg_message_t *msg = NULL;
  const gchar *contact = NULL;
  gchar *name = NULL;
  gchar *photo_path = NULL;

  msg = msg_test_get_msg_by_id (sid, msgtest.msgid);
  contact = msg_get_from (sid, msg);

  if (search_from_addressbook (0, contact, &name, &photo_path))
  {
    if (name == NULL)
      g_print
        ("*************************************************name is null \n");
    if (photo_path == NULL)
      g_print
        ("*************************************************photopath is NULL\n");
    sms_iac_dialer (contact, name, photo_path);
  }
  
  msg_release (sid, msg);
  g_free (name);
  g_free (photo_path);
  return 0;
}

static void
saveContact ()
{
  make_save_contact (NULL);
}
static void
copy_add ()
{
  make_copy_add (NULL);
}

/* send sms form folders execpt Inbox */
static int
send_from_folder ()
{
  msg_message_t *msg = NULL;
  const gchar *contact = NULL;
  gchar *name = NULL;
  gchar *name_list = NULL;
  gchar *contact_list = NULL;
  gchar *body = NULL;
  gint size = 0;

  msg = msg_test_get_msg_by_id (sid, msgtest.msgid);
  contact = msg_get_recipientlist (sid, msg);

  body = msg_get_body_by_id (sid, msgtest.msgid, &size);

  if (search_from_addressbook (0, contact, &name, NULL))
  {
    if (name)
    {
      name_list = g_strdup_printf ("%s", name);
      contact_from_addrbook = g_list_append (contact_from_addrbook, name_list);
      contact_list = g_strdup_printf ("%s", contact);
      contact_from_addrbook =
        g_list_append (contact_from_addrbook, contact_list);

      pre_list_count = g_list_length (contact_from_addrbook);

      contact = name;
    }
  }
  
  g_print ("List count:%d\n", g_list_length (contact_from_addrbook));
  make_sms_write ((gchar *) contact, body);

  msg_release (sid, msg);
  g_free (name);
  g_free (body);
  return 0;
}

static void
recycle ()
{
}

static void
clean_trash ()
{
}

static void
edit ()
{
}

static void
clean_trash_all ()
{
}

static void
exit_pop_menu ()
{
  GtkWidget *window;

  window = get_current_window ();
  gpe_skwindow_set_softkey_label (GPE_SKWINDOW (window), SK_RIGHT, "Back");
  gpe_skwindow_set_softkey_label (GPE_SKWINDOW (window), SK_LEFT, "Options");
}


/* Action description table to set up the GtkActionGroup. */
static const GtkActionEntry action_entries[] = {
  {"Send", NULL, "_Send", "", "", G_CALLBACK (send)},
  {"Save", NULL, "_Save", "", "", G_CALLBACK (save)},
  {"Insert", NULL, "_Insert", "", "", G_CALLBACK (insert)},
  {"Clean", NULL, "_Clean", "", "", G_CALLBACK (clean)},
  {"PhoneNum", NULL, "_PhoneNum", "", "", G_CALLBACK (get_addrbook)},
  {"Template", NULL, "_Template", "", "", G_CALLBACK (get_template)},
  {"View", NULL, "_View", "", "", G_CALLBACK (view)},
  {"Reply", NULL, "_Reply", "", "", G_CALLBACK (reply_sms)},
  {"Forward", NULL, "_Forward", "", "", G_CALLBACK (forward)},
  {"CallBack", NULL, "_CallBack", "", "", G_CALLBACK (callBack)},
  {"Delete", NULL, "_Delete", "", "", G_CALLBACK (delete_sms)},
  {"Search", NULL, "_Search", "", "", G_CALLBACK (search)},
  {"Sort", NULL, "_Sort", "", "", G_CALLBACK (sort)},
  {"Name", NULL, "_Name", "", "", G_CALLBACK (sort_by_name)},
  {"Date", NULL, "_Date", "", "", G_CALLBACK (sort_by_date)},
  {"MultiSel", NULL, "_MultiSel", "", "", G_CALLBACK (multiSel)},
  {"MoveTo", NULL, "_Move to", "", "", G_CALLBACK (move)},
  {"SIM", NULL, "_SIM", "", "", G_CALLBACK (move_to_SIM)},
  {"Phone", NULL, "_Phone", "", "", G_CALLBACK (move_to_phone)},
  {"SaveContact", NULL, "_SaveContact", "", "", G_CALLBACK (saveContact)},
  {"CopyAdd", NULL, "_Copy&Add", "", "", G_CALLBACK (copy_add)},
  {"Recycle", NULL, "_Recycle", "", "", G_CALLBACK (recycle)},
  {"CleanTrash", NULL, "_Clean", "", "", G_CALLBACK (clean_trash)},
  {"SendFolder", NULL, "_Send", "", "", G_CALLBACK (send_from_folder)},
  {"Edit", NULL, "_Edit", "", "", G_CALLBACK (edit)},
  {"CleanAll", NULL, "_CleanAll", "", "", G_CALLBACK (clean_trash_all)},
  {"Exit", NULL, "_Exit", "", "", G_CALLBACK (exit_pop_menu)},
};




static GtkUIManager *
create_ui_manager ()
{
  GError *err = NULL;
  gchar *file_path = NULL;
  GtkUIManager *ui_manager;
  GtkActionGroup *action_group;

  /* Set up action group and feed UI manager with actions and UI description. */
  action_group = gtk_action_group_new ("AllActions");
  ui_manager = gtk_ui_manager_new ();

  gtk_action_group_add_actions (action_group, action_entries,
                                G_N_ELEMENTS (action_entries), NULL);
  gtk_ui_manager_insert_action_group (ui_manager, action_group, 0);
  file_path = g_strdup_printf ("%s/%s", PREFIX, UI_DES);
  if (!gtk_ui_manager_add_ui_from_file (ui_manager, file_path, &err))
  {
    g_printerr ("Failed to parse menu data: %s\n", err->message);
    g_error_free (err);

  }
  g_free (file_path);

  return ui_manager;

}


static void
sms_app_init ()
{
  gchar *file_path = NULL;

  if (msg_service_init () == 0)
  {
    printf ("service started\n");

    msg_evt_register (1, 2, send_success_cb, NULL);
    msg_evt_register (1, 3, send_failed_cb, NULL);
    msg_evt_register (1, MSG_EVT_MSG_INCOMING, msg_incoming_cb, NULL);
    msg_evt_register (1, 4, status_report_cb, NULL);
    service_start = 1;

  }
  else
    printf ("failed to start service\n");

  file_path = g_strdup_printf ("%s/%s", PREFIX, RC);
  if (file_exists (file_path))
    gtk_rc_add_default_file (file_path);

  sms_flag.multi_select_flag = FALSE;
  sms_flag.sort_date_flag = FALSE;
  sms_flag.start_from_abook_flag = FALSE;
  sms_flag.view_new_flag = FALSE;
  sms_flag.dbus_invoke_flag = FALSE;
  sms_flag.create_flag = FALSE;


  sms_path.folder_path = NULL;
  sms_path.sms_list_path = NULL;
  sms_path.sms_number = 0;

  sms_treeview.folder_list_store = NULL;
  sms_treeview.folder_list_tree_view = NULL;
  sms_treeview.sms_list_store = NULL;
  sms_treeview.sms_list_tree_view = NULL;

  time_out_id = 0;
  count = 0;
  sid = 1;
  service_start = 0;

  msg_sms_free_list (contact_from_addrbook);
  msg_sms_free_hashtable (hashtable);

  hashtable = g_hash_table_new (NULL, NULL);

  ui_manager = create_ui_manager ();

  pim_init ();

  if (sms_iac_initialize ())
    g_print ("Service Register\n");

  g_free (file_path);
}

void
show_the_first_window (char *input_arg)
{
  if (input_arg == NULL)
  {
    make_msg_window ();
  }
  else if (strcmp (input_arg, "send_sms") == 0)
  {
    sms_flag.start_from_abook_flag = TRUE;
    make_sms_write (NULL, NULL);
    get_contacts_from_addrbook ();
  }
  else if (strcmp (input_arg, "inbox") == 0)
  {
    gchar *inbox_path = "1";

    make_msg_window ();
    msgtest.fid = MSG_FOLDER_SMS_ROOT;
    make_sms_window ();
    set_tree_view_cursor (sms_treeview.folder_list_tree_view, inbox_path);
  }
  else
  {
    make_msg_window ();
  }
}

gboolean
display_folders (GtkWidget *dialog)
{

  gtk_widget_destroy (dialog);
  make_sms_folder (msgtest.fid);
  return FALSE;
}

gboolean
destory_wait_phoneserver_dialog (GtkWidget *dialog)
{
  if (is_phoneserver_ready ())
  {
    g_timeout_add (1, (GSourceFunc) display_folders, dialog);
    sms_flag.open_flag = TRUE;
    ps_timeout_id = 0;
    return FALSE;
  }
  else
  {
    return TRUE;
  }

}

/* main function */
int
main (int argc, char *argv[])
{

  struct flock flock;
  gint fd = 0;

  gtk_set_locale ();
  gtk_init (&argc, &argv);

  sms_app_init ();

  is_phoneserver_ready ();

  phone_server_ready = 1;

  show_the_first_window (argv[1]);

run:
  gtk_main ();

  return 0;
}

