/*	sms - application for sending, receiving and management of short message
 *
 *	Authors: YIN Huasong, WU Yu <yu.wu@orange-ftgroup.com> 
 *	
 *	This software and associated documentation files (the "Software") 
 *	are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *	All Rights Reserved. 
 *
 *	A copyright license is hereby granted for redistribution and use of 
 *	the Software in source and binary forms, with or without modification, 
 *	provided that the following conditions are met: 
 *	- Redistributions of source code must retain the above copyright notice, 
 *	this copyright license and the following disclaimer. 
 *  - Redistributions in binary form must reproduce the above copyright 
 * 	notice, this copyright license and the following disclaimer in the 
 *	documentation and/or other materials provided with the distribution. 
 *	- Neither the name of LiPS  nor the names of its Members may be used 
 *	to endorse or promote products derived from the Software without 
 *	specific prior written permission. 
 *
 *	A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *	to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *	distribute any implementation compliant with the any specification adopted 
 *	by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *	and non-discriminatory terms and conditions and under reciprocity, as 
 *	regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *	"AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *	THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *	AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *	ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *	POSSIBILITY OF SUCH DAMAGE. 
 */

#include <glib.h>
#include <gtk/gtk.h>
#include <iac.h>

#include "shared.h"
#include "callbacks.h"
#include "sms-app.h"


extern GHashTable *hashtable;   /* save every window's handle */
extern gint array[10];          /* use in hashtable */
extern gint count;              /* use in hashtable */

extern gint time_out_id;        /* timer */
extern msg_sid_t sid;           /* session id */
extern gint service_start;      /* init service */ 


extern GList *contact_from_addrbook;


static iac_context_t *context = NULL;


/* add contacts from the dbus */
static void
sms_iac_insert_contacts_func (iac_arguments_t *arguments, gpointer user_data)
{
  gint count_abook = 0;
  gint i = 0;
  const iac_value_t *val = NULL;


  count_abook = iac_arguments_size (arguments);          /* the number of values existed in arguments */

  g_print ("Count_book:%d\n", count_abook);

  g_print ("count:%d\n", count);

  for (i = 1; i < count_abook; i++)
  {
    val = iac_arguments_get_arg (arguments, i);
    contact_from_addrbook =
      g_list_append (contact_from_addrbook,
                     g_strdup_printf ("%s", val->value.s));
  }

  get_contacts_from_addrbook ();           /* show the name of contact */
}

gboolean
sms_iac_create_sms_func (const gchar *interface, const gchar *method,
                         iac_arguments_t *arguments,
                         gpointer user_data)
{
  g_print ("%s(): entering\n", __FUNCTION__);


  if (strcmp (method, SMS_IAC_METHOD_NEW_SMS) == 0)
  {
    msg_sms_new_incoming ();
    gtk_window_present (GTK_WINDOW (get_current_window ()));
  }
  else
  {
    if (!sms_flag.start_from_abook_flag)
    {
      if (!(sms_flag.create_flag))
      {
        sms_flag.dbus_invoke_flag = TRUE;
        make_sms_pre_write ();
      }
    }
    g_print ("count:%d\n", count);
    sms_iac_insert_contacts_func (arguments, NULL);
    gtk_window_present (GTK_WINDOW (get_current_window ()));
  }

  return IAC_OK;

}


gboolean
sms_iac_initialize (void)
{
  context = iac_initialize ();
  if (context == NULL)
  {
    g_print ("%s(): iac_context initialization failure\n", __FUNCTION__);

    return IAC_ERROR;
  }

  if (iac_register_service (context, SMS_APP_SERIVCE) == IAC_ERROR)
  {
    g_print ("%s(): cannot register service\n", __FUNCTION__);

    return IAC_ERROR;
  }

  if (iac_register_service (context, SMS_IAC_SERVICE) == IAC_ERROR)
  {
    g_print ("%s(): cannot register service\n", __FUNCTION__);

    return IAC_ERROR;
  }

  if (iac_register_listener (context, SMS_IAC_OBJECT, SMS_IAC_INTERFACE,
                             (iac_listener_func) sms_iac_create_sms_func, NULL)
      != IAC_OK)
  {
    iac_deinitialize (context);
    return IAC_ERROR;
  }


  return IAC_OK;
}

void
sms_iac_finalize (void)
{
  if (context)
  {
    iac_unregister_listener (context, SMS_IAC_OBJECT, SMS_IAC_INTERFACE);
    iac_deinitialize (context);
  }

  return;
}

gboolean
sms_iac_invoked_app (void)
{
  return sms_flag.dbus_invoke_flag;
}

gboolean
sms_iac_abook (void)
{

  if (iac_start_service (context, ADDRESSBOOK_APP_SERIVCE,
                         ADDRESSBOOK_IAC_METHOD_MULTI_SEL))
  {
    g_print ("%s(): start app service %s failure\n",
             __FUNCTION__, ADDRESSBOOK_APP_SERIVCE);

    return IAC_ERROR;
  }

  g_print ("%s(): iac_execute()start\n", __FUNCTION__);
  iac_execute_service (context, ADDRESSBOOK_IAC_SERVICE,
                       ADDRESSBOOK_IAC_OBJECT,
                       ADDRESSBOOK_IAC_INTERFACE,
                       ADDRESSBOOK_IAC_METHOD_MULTI_SEL,
                       NULL, NULL);
  g_print ("%s(): iac_execute() done\n", __FUNCTION__);

  return IAC_OK;
}


/* contect to the voice call */
gboolean
sms_iac_dialer (const gchar *dialnum, const gchar *name,
                const gchar *picpath)
{
  iac_arguments_t *args = NULL;


  args = iac_arguments_new_with_args (IAC_TYPE_STRING, dialnum,
                                      IAC_TYPE_STRING, name,
                                      IAC_TYPE_STRING, picpath,
                                      IAC_TYPE_INVALID);


  if (iac_start_service (context, VOICECALL_APP_SERVICE,
                         VOICECALL_IAC_METHOD_CALLING_INVOKE))
  {
    g_print ("%s(): start app service %s failure\n",
             __FUNCTION__, VOICECALL_APP_SERVICE);

    return IAC_ERROR;
  }

  g_print ("%s(): iac_execute()start\n", __FUNCTION__);
  iac_execute_service (context, VOICECALL_IAC_SERVICE, VOICECALL_IAC_OBJECT,
                       VOICECALL_IAC_INTERFACE,
                       VOICECALL_IAC_METHOD_CALLING_INVOKE,
                       args, NULL);

  g_print ("%s(): iac_execute() done\n", __FUNCTION__);

  iac_arguments_free (args);

  return IAC_OK;
}

/*
vi:ts=2:nowrap:ai:expandtab 
*/

